/**
 * amis v2.0.1
 * Copyright 2018-2022 baidu
 */

import { __generator, __assign, __extends, __decorate } from 'tslib';
import React from 'react';
import { StoreNode, normalizeApiResponseData, isObject, guid, isPureVariable, resolveVariableAndFilter, isApiOutdated, isEffectiveApi, normalizeApi, getPropValue, Renderer } from 'amis-core';
import { withStore } from 'amis-ui';
import { types, flow } from 'mobx-state-tree';

var _a;
var Store = StoreNode.named('MappingStore')
    .props({
    fetching: false,
    errorMsg: '',
    map: types.frozen({})
})
    .actions(function (self) {
    var load = flow(function (env, api, data) {
        var ret, data_1, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 2, 3, 4]);
                    self.fetching = true;
                    return [4 /*yield*/, env.fetcher(api, data)];
                case 1:
                    ret = _a.sent();
                    if (ret.ok) {
                        data_1 = normalizeApiResponseData(ret.data);
                        self.setMap(data_1);
                    }
                    else {
                        throw new Error(ret.msg || 'fetch error');
                    }
                    return [3 /*break*/, 4];
                case 2:
                    e_1 = _a.sent();
                    self.errorMsg = e_1.message;
                    return [3 /*break*/, 4];
                case 3:
                    self.fetching = false;
                    return [7 /*endfinally*/];
                case 4: return [2 /*return*/];
            }
        });
    });
    return {
        load: load,
        setMap: function (options) {
            if (isObject(options)) {
                self.map = __assign({}, options);
            }
        }
    };
});
var MappingField = withStore(function (props) {
    return Store.create({
        id: guid(),
        storeType: Store.name
    }, props.env);
})((_a = /** @class */ (function (_super) {
        __extends(class_1, _super);
        function class_1(props) {
            var _this = _super.call(this, props) || this;
            props.store.syncProps(props, undefined, ['map']);
            return _this;
        }
        class_1.prototype.componentDidMount = function () {
            var _a = this.props; _a.store; _a.source; _a.data;
            this.reload();
        };
        class_1.prototype.componentDidUpdate = function (prevProps) {
            var props = this.props;
            var _a = this.props, store = _a.store, source = _a.source, data = _a.data;
            store.syncProps(props, prevProps, ['map']);
            if (isPureVariable(source)) {
                var prev = resolveVariableAndFilter(prevProps.source, prevProps.data, '| raw');
                var curr = resolveVariableAndFilter(source, data, '| raw');
                if (prev !== curr) {
                    store.setMap(curr);
                }
            }
            else if (isApiOutdated(prevProps.source, props.source, prevProps.data, props.data)) {
                this.reload();
            }
        };
        class_1.prototype.reload = function () {
            var _a;
            var _b = this.props, source = _b.source, data = _b.data, env = _b.env;
            var store = this.props.store;
            if (isPureVariable(source)) {
                store.setMap(resolveVariableAndFilter(source, data, '| raw'));
            }
            else if (isEffectiveApi(source, data)) {
                var api = normalizeApi(source, 'get');
                api.cache = (_a = api.cache) !== null && _a !== void 0 ? _a : 30 * 1000;
                store.load(env, api, data);
            }
        };
        class_1.prototype.renderSingleValue = function (key, reactKey) {
            var _a;
            var _b = this.props, className = _b.className, placeholder = _b.placeholder, render = _b.render, cx = _b.classnames; _b.name; _b.data; var store = _b.store;
            var viewValue = (React.createElement("span", { className: "text-muted" }, placeholder));
            var map = store.map;
            var value = undefined;
            // trim 一下，干掉一些空白字符。
            key = typeof key === 'string' ? key.trim() : key;
            if (typeof key !== 'undefined' &&
                map &&
                (value =
                    (_a = map[key]) !== null && _a !== void 0 ? _a : (key === true && map['1']
                        ? map['1']
                        : key === false && map['0']
                            ? map['0']
                            : map['*'])) !== undefined) {
                viewValue = render('tpl', value);
            }
            return (React.createElement("span", { key: "map-".concat(reactKey), className: cx('MappingField', className) }, viewValue));
        };
        class_1.prototype.render = function () {
            var _this = this;
            var mapKey = getPropValue(this.props);
            if (Array.isArray(mapKey)) {
                return (React.createElement("span", null, mapKey.map(function (singleKey, index) {
                    return _this.renderSingleValue(singleKey, index);
                })));
            }
            else {
                return this.renderSingleValue(mapKey, 0);
            }
        };
        return class_1;
    }(React.Component)),
    _a.defaultProps = {
        placeholder: '-',
        map: {
            '*': '通配值'
        }
    },
    _a));
/** @class */ ((function (_super) {
    __extends(MappingFieldRenderer, _super);
    function MappingFieldRenderer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MappingFieldRenderer.prototype.render = function () {
        return React.createElement(MappingField, __assign({}, this.props));
    };
    MappingFieldRenderer = __decorate([
        Renderer({
            test: /(^|\/)(?:map|mapping)$/,
            name: 'mapping'
        })
    ], MappingFieldRenderer);
    return MappingFieldRenderer;
})(React.Component));

export { MappingField, Store };
